var type = require('./type.js');
var clone = require('./clone.js');
var assert = require('./assert.js');
var object = require('../share/util/object.js');

function emptyFn() {}

function mixin(target, mixins) {
    object.each(mixins, function(value, key) {
        this[key] = value;
    }, target);
    return target;
}

function addProperties(target, mixins, deep) {
    if (!type.isDefined(deep)) {
        deep = false;
    }
    object.each(mixins, function(value, key) {
        if (!type.isFunction(value)) {
            if (deep) {
                this[key] = clone(value);
            } else {
                this[key] = value;
            }
        }
    }, target);
    return target;
}

function addMethods(target, mixins) {
    object.each(mixins, function(method, name) {
        if (type.isFunction(method)) {
            this[name] = method;
        }
    }, target);
    return target;
}

function define(superclazz, data, overrideInitialChain) {
    if (type.isObject(superclazz) && (!type.isDefined(data) || type.isBoolean(data))) {
        // none superclazz, default is Object
        overrideInitialChain = data;
        data = superclazz;
        superclazz = Object;
    }
    data = type.isObject(data) ? data : {};
    assert(type.isFunction(superclazz) && type.isObject(superclazz.prototype), 'superclazz should be a subclass of Object');
    data.constructor = type.isFunction(data.constructor) ? data.constructor : emptyFn;
    function Clazz() {
        if (!overrideInitialChain) {
            superclazz.apply(this, arguments);
        }
        addProperties(this, data, true);
        data.constructor.apply(this, arguments);
    };
    Clazz.prototype = Object.create(superclazz.prototype);
    Clazz.prototype.constructor = data.constructor;
    Clazz.prototype.equals = function(obj) {
        return this === obj;
    };
    addMethods(Clazz.prototype, data);
    Clazz.prototype.super = function() {
        return superclazz.prototype;
    };
    Clazz.prototype.superclazz = function() {
        return superclazz;
    };
    Clazz.prototype.clazz = function() {
        return Clazz;
    };
    return Clazz;
}

module.exports = {
    mixin: mixin,
    define: define,
    emptyFn: emptyFn
};