var type = require('../core/type.js');
var clazz = require('../core/clazz.js');
var assert = require('../core/assert.js');

var Observable = clazz.define({
    signals: {},
    constructor: function Observable() {},
    clear: function() {
        this.signals = {};
    },
    emit: function() {
        var args = Array.prototype.slice.call(arguments),
            channel = args.shift(),
            i, len;
        if (this.signals[channel]) {
            for (i = 0, len = this.signals[channel].length; i < len; i++) {
                try {
                    this.signals[channel][i].apply(this, args);
                } catch(e) {
                    console.log(e);
                }
            }
        }
    },
    on: function(channel, handler) {
        assert(type.isFunction(handler), 'A handler is required.');
        if (!type.isDefined(this.signals[channel])) {
            this.signals[channel] = [];
        }
        this.signals[channel].push(handler);
        return this;
    },
    off: function(channel, handler) {
        var i, len, fn;
        if (type.isDefined(this.signals[channel])) {
            for (i = 0, len = this.signals[channel].length; i < len; i++) {
                if (this.signals[channel][i] === handler) {
                    this.signals[channel].splice(i, 1);
                }
            }
        }
        return this;
    }
});

module.exports = Observable;
