var clazz = require('../../core/clazz.js');
var type = require('../../core/type.js');
var array = require('../util/array.js');

var Collection = clazz.define({
    bucket: [],
    constructor: function Collection(v) {
        this.addAll(v);
    },
    clone: function() {
        return new Collection(this);
    },
    add: function(item) {
        this.bucket.push(item);
        return this;
    },
    addAll: function(v) {
        if (type.is(v, Collection)) {
            v.each(function(item) {
                this.add(item);
            }, this);
        } else if (type.isArray(v)) {
            array.each(v, function(item) {
                this.add(item);
            }, this);
        }
        return this;
    },
    remove: function(item) {
        var index = array.indexOf(item, this.bucket);
        if (index === -1) {
            return;
        }
        return this.bucket.splice(index, 1).shift();
    },
    removeAll: function(collection) {
        collection.each(function(item) {
            this.remove(item);
        }, this);
    },
    clear: function() {
        this.bucket = [];
    },
    size: function() {
        return this.bucket.length;
    },
    isEmpty: function() {
        return this.size() === 0;
    },
    contains: function(item) {
        return array.indexOf(item, this.bucket) !== -1;
    },
    each: function(callback, scope) {
        array.each(this.bucket, callback, scope);
    },
    filter: function(filter, scope) {
        return array.filter(this.bucket, filter, scope);
    },
    toArray: function() {
        return this.clone().bucket;
    },
    intersection: function(collection) {
        var bigger = this, smaller = collection;
        if (collection.size() > this.size()) {
            bigger = collection;
            smaller = this;
        }
        return smaller.filter(function(item) {
            return this.contains(item);
        }, bigger);
    },
    union: function(collection) {
        return this.clone().addAll(collection).toArray();
    },
    difference: function(collection) {
        return this.filter(function(item) {
            return !this.contains(item);
        }, collection);
    }
});

module.exports = Collection;