var clazz = require('../../core/clazz.js');
var type = require('../../core/type.js');
var array = require('../util/array.js');
var Collection = require('./Collection.js');
var RedBlackTree = require('../tree/RedBlackTree.js');

var TreeSet = clazz.define(Collection, {
    constructor: function TreeSet(collection, compare) {
        this.bucket = new RedBlackTree(compare);
        this.addAll(collection);
    },
    clone: function() {
        return new TreeSet(this, this.bucket.compare);
    },
    add: function(item) {
        if (!this.contains(item)) {
            this.bucket.insert(item);
        }
        return this;
    },
    remove: function(item) {
        return this.bucket.delete(item);
    },
    clear: function() {
        this.bucket = new RedBlackTree(this.bucket.compare);
    },
    size: function() {
        return this.bucket.size;
    },
    isEmpty: function() {
        return this.bucket.size === 0;
    },
    contains: function(item) {
        return !this.bucket.get(item).isNil();
    },
    each: function(callback, scope) {
        this.bucket.each(callback, scope);
    },
    filter: function(filter, scope) {
        return this.bucket.map(filter, scope);
    },
    toArray: function() {
        return this.bucket.map();
    }
}, true);

module.exports = TreeSet;