var type = require('../../core/type.js');
var hasOwnProperty = Object.prototype.hasOwnProperty,
    hasDontEnumBug = !({ toString: null }).propertyIsEnumerable('toString'),
    dontEnums = [
      'toString',
      'toLocaleString',
      'valueOf',
      'hasOwnProperty',
      'isPrototypeOf',
      'propertyIsEnumerable',
      'constructor'
    ],
    dontEnumsLength = dontEnums.length;
module.exports = {
    // From https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/keys
    keys: function(obj) {
        var keys = [], key, i;
        for (key in obj) {
            if (hasOwnProperty.call(obj, key)) {
                keys.push(key);
            }
        }
        if (hasDontEnumBug) {
            for (i = 0; i < dontEnumsLength; i++) {
                if (hasOwnProperty.call(obj, dontEnums[i])) {
                    keys.push(dontEnums[i]);
                }
            }
        }
        return keys;
    },
    values: function(obj) {
        var values = [], key, i;
        for (key in obj) {
            if (hasOwnProperty.call(obj, key)) {
                values.push(obj[key]);
            }
        }
        if (hasDontEnumBug) {
            for (i = 0; i < dontEnumsLength; i++) {
                if (hasOwnProperty.call(obj, dontEnums[i])) {
                    values.push(obj[dontEnums[i]]);
                }
            }
        }
        return values;
    },
    each: function(obj, callback, scope) {
        var key, i;
        if (!type.isObject(scope)) {
            scope = obj;
        }
        for (key in obj) {
            if (hasOwnProperty.call(obj, key)) {
                callback.call(scope, obj[key], key);
            }
        }
        if (hasDontEnumBug) {
            for (i = 0; i < dontEnumsLength; i++) {
                if (hasOwnProperty.call(obj, dontEnums[i])) {
                    callback.call(scope, obj[dontEnums[i]], dontEnums[i]);
                }
            }
        }
    },
    filter: function(obj, filter, scope) {
        var key, i, filtered = {};
        if (!type.isObject(scope)) {
            scope = obj;
        }
        for (key in obj) {
            if (hasOwnProperty.call(obj, key) && filter.call(scope, obj[key], key) !== false) {
                filtered[key] = obj[key];
            }
        }
        if (hasDontEnumBug) {
            for (i = 0; i < dontEnumsLength; i++) {
                if (hasOwnProperty.call(obj, dontEnums[i]) && filter.call(scope, obj[dontEnums[i]], dontEnums[i]) !== false) {
                    filtered[key] = obj[key];
                }
            }
        }
        return filtered;
    }
};